import 'package:rxdart/rxdart.dart';

/// A singleton class that provides behavior subjects for phone number input.
///
/// This class provides behavior subjects for managing phone number input,
/// including the country code and the phone number itself. It is designed
/// as a singleton to ensure consistent access to the behavior subjects
/// throughout the application. The behavior subjects are initialized with
/// default values.
class PhoneInputBehaviorSubject {
  /// Creates an instance of [PhoneInputBehaviorSubject].
  factory PhoneInputBehaviorSubject() {
    return _singleton;
  }

  PhoneInputBehaviorSubject._internal();
  static final PhoneInputBehaviorSubject _singleton =
      PhoneInputBehaviorSubject._internal();

  BehaviorSubject<String> countryCode = BehaviorSubject<String>.seeded('+254');
  BehaviorSubject<String> phoneNumber = BehaviorSubject<String>();
}
