import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';
import 'package:sghi_core/afya_moja_core/src/app_asset_strings.dart';
import 'package:sghi_core/afya_moja_core/src/widget_keys.dart';

class HeroImageWrapper extends StatelessWidget {
  const HeroImageWrapper(
      {super.key, required this.contentDetails, required this.isVideo});

  final Content contentDetails;
  final bool isVideo;

  @override
  Widget build(BuildContext context) {
    final bool hasHeroImage = contentDetails.heroImage != null &&
        contentDetails.heroImage?.url != UNKNOWN &&
        (contentDetails.heroImage?.url?.isNotEmpty ?? false);

    return Stack(
      alignment: Alignment.center,
      children: <Widget>[
        if (hasHeroImage) ...<Widget>[
          LeadingGraphicWidget(
            contentDetails: contentDetails,
            heroImage: Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: <Widget>[
                Expanded(
                  child: GalleryImageWidget(
                    borderRadius: const BorderRadius.only(
                      topLeft: Radius.circular(12),
                      topRight: Radius.circular(12),
                    ),
                    imageUrl: contentDetails.heroImage?.url ?? '',
                    height: 500,
                  ),
                ),
              ],
            ),
          )
        ] else ...<Widget>[
          Stack(
            key: contentHeroImagePlaceHolder,
            children: <Widget>[
              DecoratedBox(
                decoration: BoxDecoration(
                  image: const DecorationImage(
                    image: AssetImage(
                      contentImagePlaceHolderSvgPath,
                    ),
                    fit: BoxFit.cover,
                  ),
                  color: Theme.of(context).primaryColor.withOpacity(0.3),
                  borderRadius: const BorderRadius.only(
                    topRight: Radius.circular(12),
                    topLeft: Radius.circular(12),
                  ),
                ),
              ),
              Positioned(
                child: DecoratedBox(
                  decoration: BoxDecoration(
                    color: Colors.black.withOpacity(0.6),
                    borderRadius: const BorderRadius.only(
                      topRight: Radius.circular(12),
                      topLeft: Radius.circular(12),
                    ),
                  ),
                ),
              ),
              Center(
                child: SvgPicture.asset(
                  contentIconSvgPath,
                  colorFilter:
                      const ColorFilter.mode(marigoldNavColor, BlendMode.srcIn),
                ),
              )
            ],
          )
        ],
        // A video playback icon if there is a video
        if (isVideo)
          SizedBox(
            key: feedVideoPlayIconKey,
            child: SvgPicture.asset(playIcon, width: 50, height: 50),
          ),
      ],
    );
  }
}
