import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';
import 'package:sghi_core/afya_moja_core/src/app_asset_strings.dart';
import 'package:sghi_core/afya_moja_core/src/app_strings.dart';
import 'package:sghi_core/user_feed/src/domain/value_objects/colors.dart';

class PDFWrapper extends StatelessWidget {
  const PDFWrapper({
    super.key,
    required this.contentDetails,
    required this.shouldShowReactions,
  });

  final Content contentDetails;
  final bool shouldShowReactions;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: !shouldShowReactions
          ? const EdgeInsets.symmetric(horizontal: 12)
          : null,
      decoration: BoxDecoration(
        borderRadius: !shouldShowReactions ? BorderRadius.circular(10) : null,
        color: primaryPurpleColor.withOpacity(0.1),
      ),
      padding: const EdgeInsets.all(13),
      child: Row(
        children: <Widget>[
          SizedBox(
            height: 100,
            width: 100,
            child: DecoratedBox(
              decoration: BoxDecoration(
                color: primaryPurpleColor.withOpacity(0.2),
                borderRadius: const BorderRadius.all(Radius.circular(10)),
              ),
              child: Stack(
                children: <Widget>[
                  Center(
                    child: SvgPicture.asset(
                      pdfIconSvg,
                      width: 70,
                      height: 70,
                      colorFilter: const ColorFilter.mode(
                        primaryPurpleColor,
                        BlendMode.srcIn,
                      ),
                    ),
                  ),
                  Positioned(
                    top: 0,
                    right: 0,
                    child: Container(
                      decoration: const BoxDecoration(
                        borderRadius: BorderRadius.all(Radius.circular(8)),
                        color: primaryPurpleColor,
                      ),
                      padding: const EdgeInsets.symmetric(
                        horizontal: 4,
                        vertical: 3,
                      ),
                      margin: const EdgeInsets.all(8),
                      child: Text(
                        pdfText,
                        style: normalSize12Text(Colors.white),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
          size15HorizontalSizedBox,
          Expanded(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Align(
                  alignment: Alignment.topLeft,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: <Widget>[
                      Flexible(
                        child: Text(
                          contentDetails.title ?? '',
                          maxLines: 1,
                          overflow: TextOverflow.ellipsis,
                          style: boldSize16Text(
                            myAfyaHubSecondaryColor,
                          ),
                        ),
                      ),
                      Padding(
                        padding: const EdgeInsets.only(left: 8.0),
                        child: humanizeDate(
                          dateTextStyle: normalSize12Text(
                            greyTextColor,
                          ),
                          loadedDate: contentDetails.metadata?.createdAt ?? '',
                        ),
                      ),
                    ],
                  ),
                ),
                verySmallVerticalSizedBox,
                if (contentDetails.authorName != null)
                  Text(
                    contentDetails.authorName!,
                    maxLines: 1,
                    overflow: TextOverflow.ellipsis,
                    style: boldSize12Text(
                      greyTextColor,
                    ),
                  ),
                // Reactions
                if (shouldShowReactions)
                  Padding(
                    padding: const EdgeInsets.only(top: 18, bottom: 4),
                    child: Row(
                      children: <Widget>[
                        ReactionItem(
                          iconUrl: heartIconUrl,
                          count: contentDetails.likeCount,
                        ),
                        ReactionItem(
                          iconUrl: shareIconUrl,
                          count: contentDetails.shareCount,
                        ),
                        ReactionItem(
                          iconUrl: saveIconUrl,
                          count: contentDetails.bookmarkCount,
                        ),
                      ],
                    ),
                  ),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
