import 'package:flutter/material.dart';

import 'package:sghi_core/afya_moja_core/src/presentation/onboarding_scaffold_header.dart';
import 'package:sghi_core/afya_moja_core/src/presentation/responsive_widget.dart';
import 'package:sghi_core/afya_moja_core/src/presentation/spaces.dart';

/// [OnboardingScaffold] a scaffold used during the onboarding process
class OnboardingScaffold extends StatelessWidget {
  const OnboardingScaffold({
    required this.child,
    required this.title,
    required this.description,
    this.backgroundColor,
    this.showBackButton,
  });

  final Color? backgroundColor;
  final Widget child;
  final String description;
  // The title of the scaffold
  final String title;
  final bool? showBackButton;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: showBackButton ?? false
          ? AppBar(
              backgroundColor: Colors.white,
              elevation: 0,
              leading: IconButton(
                onPressed: () {
                  if (Navigator.of(context).canPop()) {
                    Navigator.of(context).pop();
                  }
                },
                icon: const Icon(Icons.arrow_back_ios, color: Colors.black),
              ),
            )
          : null,
      backgroundColor: backgroundColor,
      body: SizedBox.expand(
        child: SingleChildScrollView(
          physics: const BouncingScrollPhysics(),
          child: SafeArea(
            child: Padding(
              padding: EdgeInsets.symmetric(
                horizontal: ResponsiveWidget.preferredPaddingOnStretchedScreens(
                  context: context,
                ),
              ),
              child: (title.isNotEmpty || description.isNotEmpty)
                  ? Column(
                      mainAxisAlignment: MainAxisAlignment.center,
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        const SizedBox(height: 40),
                        OnboardingScaffoldHeader(
                          title: title,
                          description: description,
                        ),
                        smallVerticalSizedBox,
                        child,
                      ],
                    )
                  : child,
            ),
          ),
        ),
      ),
    );
  }
}
