/// Signature for a callback function that takes a [String] parameter.
///
/// This typedef defines the signature for a callback function that takes a single
/// [String] parameter and returns `void`. It is often used to represent functions
/// that are invoked when a string value changes.
///
/// @param String The string parameter of the callback.
typedef OnChangedString = void Function(String);

/// Signature for a phone number formatting function.
///
/// This typedef defines the signature for a phone number formatting function.
/// The function takes two required parameters: [countryCode] and [phoneNumber],
/// both of type [String]. It is expected to return a formatted phone number string.
/// This function is often used for formatting phone numbers based on a given
/// country code and phone number.
///
/// @param countryCode The country code of the phone number.
/// @param phoneNumber The raw phone number to be formatted.
/// @return A formatted phone number string.
typedef PhoneNumberFormatterFunc = String Function({
  required String countryCode,
  required String phoneNumber,
});
