import 'package:flutter/material.dart';
import 'package:sghi_core/ui_components/src/small_appbar.dart';
import 'package:sghi_core/user_feed/src/domain/entities/link.dart';
import 'package:sghi_core/user_feed/src/domain/value_objects/strings.dart';
import 'package:sghi_core/user_feed/src/domain/value_objects/widget_keys.dart';
import 'package:sghi_core/user_feed/src/presentation/image_viewer/image_viewer.dart';

class FeedItemImageGridPage extends StatelessWidget {
  const FeedItemImageGridPage({super.key, required this.images});

  final List<Link?>? images;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: feedImageGridPageKey,
      appBar: const SILSmallAppBar(title: pageTitleForImages),
      body: (images == null || images!.isEmpty)
          ? Container(key: noImagesContainer)
          : Container(
              padding: const EdgeInsets.symmetric(vertical: 15),
              child: GridView.builder(
                gridDelegate: const SliverGridDelegateWithFixedCrossAxisCount(
                  crossAxisCount: 4, // Adjust the number of columns here
                  mainAxisSpacing: 5.0,
                  crossAxisSpacing: 5.0,
                ),
                itemCount: images!.length,
                shrinkWrap: true,
                physics: const BouncingScrollPhysics(),
                itemBuilder: (BuildContext context, int index) {
                  final String imageUrl = images![index]!.url!;
                  final String imageID = images![index]!.id!;

                  return GestureDetector(
                    key: Key(imageID),
                    onTap: () {
                      Navigator.of(context).push(
                        MaterialPageRoute<FeedImageViewer>(
                          builder: (_) => FeedImageViewer(imageUrl: imageUrl),
                        ),
                      );
                    },
                    child: Container(
                      alignment: Alignment.center,
                      decoration: BoxDecoration(
                        color: Colors.white,
                        image: DecorationImage(
                          fit: BoxFit.cover,
                          image: NetworkImage(imageUrl),
                        ),
                        borderRadius: BorderRadius.circular(5.0),
                        boxShadow: const <BoxShadow>[
                          BoxShadow(
                            color: Colors.black54,
                            blurRadius: 0.3, // shadow direction: bottom right
                          )
                        ],
                      ),
                    ),
                  );
                },
              ),
            ),
    );
  }
}
