// Flutter imports:
import 'package:flutter/material.dart';

// Package imports:
import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail_image_network/mocktail_image_network.dart';
import 'package:photo_view/photo_view.dart';

// Project imports:
import 'package:sghi_core/user_feed/src/domain/entities/link.dart';
import 'package:sghi_core/user_feed/src/domain/value_objects/widget_keys.dart';
import 'package:sghi_core/user_feed/src/presentation/image_viewer/image_grid_page.dart';
import 'package:sghi_core/user_feed/src/presentation/image_viewer/image_viewer.dart';
import '../../../mock_data.dart';
import '../../../test_helpers.dart';

void main() {
  group('FeedItemImageGridPage', () {
    testWidgets('should render images grid correctly',
        (WidgetTester tester) async {
      await mockNetworkImages(() async {
        await buildTestWidget(
          tester: tester,
          child: FeedItemImageGridPage(images: <Link>[mockImageLink]),
        );
      });

      await tester.pumpAndSettle();

      // verify ui renders correctly
      expect(find.byKey(feedImageGridPageKey), findsOneWidget);
      expect(find.byType(GridView), findsOneWidget);

      final Finder imageFinder =
          find.byKey(const Key('1lmN7WFfyzVZ27bar4mh5ffAP2K'));

      expect(imageFinder, findsOneWidget);
    });

    testWidgets('should navigate to FeedImageViewer to show the images ',
        (WidgetTester tester) async {
      await mockNetworkImages(() async {
        await buildTestWidget(
          tester: tester,
          child: FeedItemImageGridPage(images: <Link>[mockImageLink]),
        );
      });

      await tester.pumpAndSettle();

      // verify ui renders correctly
      expect(find.byKey(feedImageGridPageKey), findsOneWidget);

      final Finder imageFinder =
          find.byKey(const Key('1lmN7WFfyzVZ27bar4mh5ffAP2K'));

      expect(imageFinder, findsOneWidget);

      await tester.tap(imageFinder);

      await tester.pump();
      await tester.pump();

      expect(find.byType(FeedImageViewer), findsOneWidget);
      expect(find.byType(InteractiveViewer), findsOneWidget);
      expect(find.byType(PhotoView), findsOneWidget);
    });

    testWidgets('should render empty container when there are no images',
        (WidgetTester tester) async {
      await buildTestWidget(
        tester: tester,
        child: const FeedItemImageGridPage(images: <Link>[]),
      );

      await tester.pumpAndSettle();

      expect(find.byKey(feedImageGridPageKey), findsOneWidget);
      expect(find.byKey(noImagesContainer), findsOneWidget);
    });
  });
}
