// Dart imports:
import 'dart:io';

// Flutter imports:
import 'package:flutter/material.dart';
// Package imports:
import 'package:flutter_test/flutter_test.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';
import 'package:sghi_core/user_feed/src/domain/entities/item.dart';
import 'package:sghi_core/user_feed/src/domain/entities/link.dart';
import 'package:sghi_core/user_feed/src/domain/value_objects/enums.dart';
// Project imports:
import 'package:sghi_core/user_feed/src/presentation/widgets/feed_item_detail_view.dart';

import '../../../mock_data.dart';

void main() {
  setUpAll(() => HttpOverrides.global = null);
  group('Feed item detail view', () {
    testWidgets('widgets are rendered correctly with text plain',
        (WidgetTester tester) async {
      await tester.runAsync(() async {
        await tester.pumpWidget(MaterialApp(home: Material(
          child: Scaffold(
            body: Builder(builder: (BuildContext context) {
              return FeedItemContentView(
                feedItem: mockFeedItem,
                itemTextType: TextType.PLAIN,
                summary: '---------------This is a summary',
                text: feedItemBody,
                videos: emptyMockFeedLinks,
                featureImage: '',
              );
            }),
          ),
        )));
        expect(find.byType(FeedItemContentView), findsOneWidget);
      });
    });

    testWidgets('widgets are rendered correctly with html',
        (WidgetTester tester) async {
      await tester.runAsync(() async {
        await tester.pumpWidget(MaterialApp(home: Material(
          child: Scaffold(
            body: Builder(builder: (BuildContext context) {
              return FeedItemContentView(
                feedItem: mockFeedItem,
                itemTextType: TextType.HTML,
                summary: summary,
                text: '<a href="https://github.com">Github</a>',
                videos: emptyMockFeedLinks,
                featureImage: 'https://github.com',
              );
            }),
          ),
        )));
        expect(find.byType(FeedItemContentView), findsOneWidget);
      });
    });

    testWidgets('onBodyLinkOrImageTapCallback calls launchUrl',
        (WidgetTester tester) async {
      await tester.pumpWidget(MaterialApp(home: Material(
        child: Scaffold(
          body: Builder(builder: (BuildContext context) {
            return FeedItemContentView(
              feedItem: mockFeedItem,
              itemTextType: TextType.HTML,
              summary: summary,
              text: '<a href="https://github.com">Github</a>',
              videos: notEmptyMockFeedLinks,
              featureImage: '',
            );
          }),
        ),
      )));

      expect(find.byType(FeedItemContentView), findsOneWidget);

      await tester.tap(find.text('Github'));

      await tester.pumpAndSettle();
    });

    testWidgets('should navigate back when close icon is tapped',
        (WidgetTester tester) async {
      await tester.pumpWidget(MaterialApp(
        home: Material(
          child: Scaffold(
            body: Builder(builder: (BuildContext context) {
              return Center(
                child: MyAfyaHubPrimaryButton(
                  onPressed: () {
                    Navigator.of(context).push(
                      MaterialPageRoute<FeedItemContentView>(
                        builder: (_) => FeedItemContentView(
                          text: 'Test',
                          summary: summary,
                          itemTextType: TextType.HTML,
                          feedItem: Item.fromJson(mockItemData),
                          videos: const <Link?>[],
                          featureImage: '',
                        ),
                      ),
                    );
                  },
                ),
              );
            }),
          ),
        ),
      ));

      expect(find.byType(MyAfyaHubPrimaryButton), findsOneWidget);
      await tester.tap(find.byType(MyAfyaHubPrimaryButton));
      await tester.pumpAndSettle();

      expect(find.byType(FeedItemContentView), findsOneWidget);
      expect(find.byKey(const Key('feedItemDetailBackButton')), findsOneWidget);

      await tester.tap(find.byKey(const Key('feedItemDetailBackButton')));

      await tester.pumpAndSettle();
      expect(find.byType(MyAfyaHubPrimaryButton), findsOneWidget);
    });
  });
}
